;;;; url.scm
;;;; url-compatible code for uri.scm [Daishi Kato]

(use srfi-1 regex)
(use uri)

(eval-when (compile)
  (declare
    (usual-integrations)
    (fixnum)
    (inline)
    (no-procedure-checks)
    (export
      ;; url egg compatibility (deprecated)
      url? url-scheme url-user url-password url-host url-port url-path
      url-typecode url->string url url-decode url-encode make-url) ) )

;;;
;;; URL Compatibility (Deprecated)
;;;

;Copyright (c) 2003-2005, Felix L. Winkelmann
;All rights reserved.
;
;Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
;conditions are met:
;
;  Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;    disclaimer.
;  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;    disclaimer in the documentation and/or other materials provided with the distribution.
;  Neither the name of the author nor the names of its contributors may be used to endorse or promote
;    products derived from this software without specific prior written permission.
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
;OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
;AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
;CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
;CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
;SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
;OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
;POSSIBILITY OF SUCH DAMAGE.

(define url? uri?)

(define url->string uri->string)

(define (url-scheme x)
  (and-let* ([sch (uri-scheme x)])
    (symbol->string sch) ) )

(define url-user uri-username)

(define url-password uri-password)

(define url-host uri-host)

(define url-port uri-port)

(define url-decode uri-decode-query-item)

(define (url-encode str . special)
  (if (null? special)
    (uri-encode str)
    (uri-encode str (list->string (car special)))) )

(define get-typecode
  (let ([typecode-regex (regexp ";type=(.+)")])
    (lambda (x)
      (and-let* ([lst (string-match typecode-regex (last x))])
        (string-ref (second lst) 0) ) ) ) )

(define (url-path x)
  (and-let* ([path (uri-path x)])
    (if (null? path) ; Should never happen
      #f
      (let ([path (if (get-typecode path) (drop-right path 1) path)])
        (if (null? path) ; Should never happen
          #f
          (string-intersperse (cdr path) "/"))) ) ) )

(define (url-typecode x)
  (and-let* ([cod (get-typecode (uri-path x))])
    cod ) )

(define (make-url #!key scheme user password host port path typecode)
  (let ([scheme (and scheme (string-ci->symbol scheme))]
        [authority
          (list
            (and user (if password (string-append user ":" password) user))
            host
            port)]
        [path-list (and path (uri-split-path path))]
        [option (and typecode (string-append ";type=" (string typecode)))])
    (when option
      (set! path-list (append! (or path-list '()) (list option))) )
    (*make-uri scheme (list authority path-list #f) #f)) )

;;Note - preserves original "+" <-> " " mapping behavior
;;& typecode
(define url
  (let* (
      [name-chars "[A-Za-z0-9.+%-]+"]
      [path-chars "[A-Za-z0-9.+%~/-]+"]
      [url-regex
        (regexp
          (string-append
            "((" name-chars "\\:)?(//))?(" 	; scheme
            name-chars "(\\:"               ; user
            name-chars ")?@)?(" 		        ; password
            name-chars ")?(\\:[0-9]+)?(/" 	; host
            path-chars ")?"			            ; path
            "(;type=[aid])?")) ] )          ; typecode
    (lambda (str . default)
      (match (string-match url-regex str)
        [(_ _ scheme _ user password host port path typecode)
          (make-url
            #:scheme
              (and scheme (substring scheme 0 (sub1 (string-length scheme))))
            #:user
              (and user
                   (url-decode
                     (if password
                       (substring user 0 (- (string-length user) (string-length password) 1))
                       (substring user 0 (- (string-length user) 1)) ) ) )
            #:password
              (and password (url-decode (substring password 1 (string-length password))))
            #:host
              (and host (url-decode host))
            #:port
              (and port (string->number (url-decode (substring port 1 (string-length port)))))
            #:path
              (and path (url-decode (substring path 1 (string-length path))))
            #:typecode
              (and typecode (string-ref (url-decode typecode) (sub1 (string-length typecode)))) ) ]
        [_
          (if (pair? default)
            (car default)
            (error 'url "invalid URL" str) ) ] ) ) ) )


